part of sjs;

class Schema extends Jsonbject {
  final Schema owner;
  
  Schema (Map json, [Schema owner = null]) :
    this.owner = owner,
    super(json) {
    var keys = json.keys;
    if (keys.every((key) => key is String)) {
      //parsing id
      String id = this.id;
      if (owner == null) {
        if (id == null) {
          id = '/root';
        } else {
          if (!id.startsWith('/')) {
            id = '/$id';
          }
        }
      } else {
        if (id == null) {
          id = '${owner.id}/_';
        } else {
          id = '${owner.id}/$id';
        }
      }
      this.id = id;
      _schemas[id] = this;
      
      _parse('fields')._parse('formats')._parse('types');
    } else {
      throw new ArgumentError('All keys must be String');
    }
  }
  
  Schema.fromFileSync (File file) :
    this.fromString (file.readAsStringSync());
  
  Schema.fromString (String json) :
    this(parse(json));
  
  Map<String, Field> get fields => map[KEY_FIELDS];
  
  Schema get format => map[KEY_FORMAT];
  
  Map<String, Schema> get formats => map[KEY_FORMATS];

  String get id => map[KEY_ID];
  
  String get type => map[KEY_TYPE];
  
  Map<String, Schema> get types => map[KEY_TYPES];
  
  Schema _parse (String name) {
    if (map.containsKey(name)) {
      var values = map[name];
      if (values is Map) {
        values.keys.forEach((key) {
          var value = values[key];
          if (value is Map) {
            if (name == 'fields') {
              fields[key] = new Field(value, this);
            } else {
              values[key] = new Schema(value, this);
            }
          } else {
            throw new ArgumentError('value must be Map');
          }
        });
      } else {
        throw new ArgumentError('values must be Map');
      }
    }
    return this;
  }
  
  bool validate (Map json) {
    var keys = json.keys;
    for (String key in keys) {
      if (fields.containsKey(key)) {
        if (!validateField(json[key], fields[key])) {
          return false;
        }
      } else {
        return false;
      }
    }
    return true;
  }
  
  bool validateField (value, Field field) {
    if (value == null && !field.required) {
      return true;
    }
    
    var typeName = field.type;
    switch (typeName) {
      case T_ARRAY:
        return SJSType.ARRAY.validate(value, field.map);
        break;
      case T_BOOL:
        return SJSType.BOOL.validate(value);
        break;
      case T_INT:
        return SJSType.INT.validate(value, field.map);
        break;
      case T_ENUM:
        return SJSType.ENUM.validate(value, field.map);
        break;
      case T_NULL:
        return SJSType.NULL.validate(value, field.map);
        break;
      case T_NUM:
        return SJSType.NUM.validate(value, field.map);
        break;
      case T_OBJECT:
        return SJSType.OBJECT.validate(value, field.map);
        break;
      case T_STRING:
        return SJSType.STRING.validate(value, field.map);
        break;
      default:
        if (value is Map) {
          var types = this.types;
          if (types.containsKey(typeName)) {
            Schema typeSchema = types[typeName];
            bool valid = true;
            
            value.forEach((key, val) {
              valid = valid && typeSchema.validateField(val, typeSchema.fields[key]);
            });
            return valid;
          } else {
            throw new ArgumentError('Type $typeName not found');
          }
        } else {
          throw new ArgumentError('map expected!');
        }
        break;
    }
  }
  
  set id (String id) => map[KEY_ID] = id;
  
  set fields (Map<String, Field> fields) => map[KEY_FIELDS] = fields;
  
  set format (Schema format) => map[KEY_FORMAT] = format;
  
  set formats (Map<String, Schema> formats) => map[KEY_FORMATS] = formats;
  
  set type (String type) => map[KEY_TYPE] = type;
  
  set types (Map<String, Schema> types) => map[KEY_TYPES] = types;
  
  static final Map<String, Schema> _schemas = new Map<String, Schema> ();
  
  static Map<String, Schema> get schemas => new Map.from(_schemas);
}