import 'package:unittest/unittest.dart';
import '../lib/sjs.dart';

void main () {
  test ('Dart Web Server schema test', () {
    var json = {
      'id': 'dws',
      'fields': {
        'init': {
          'type' : 'Init',
          'required': true
        }
      },
      'types': {
        'Init': {
          'type': 'object',
          'fields': {
            'address': {
              'format': ['url', 'ipv4'],
              'required': true,
              'type': 'string'
            },
            'port': {
              'max': 65535,
              'min': 0,
              'required': true,
              'type': 'int'
            }
          }
        }
      }
    };
    Schema s = new Schema(json);
    
    var config = {
      'init': {
        'address': '192.168.0.101',
        'port': 9990
      }
    };
    
    expect (s.validate(config), isTrue);
  });
  
  test ('Schema array test', () {
    var json = {
      'id': 'array_test',
      'type': 'object',
      'fields': {
        'array': {
          'type': 'array'
        }
      }
    };
    
    var s = new Schema(json);
    
    expect (s.validate({
      'array': 15
    }), isFalse);
    expect (s.validate({
      'array': []
    }), isTrue);
    expect (s.validate({
      'array': [15]
    }), isTrue);
    
    json['fields']['array']['min'] = 2;
    json['fields']['array']['max'] = 5;
    expect (s.validate({
      'array': []
    }), isFalse);
    expect (s.validate({
      'array': [15,13]
    }), isTrue);
    expect (s.validate({
      'array': [15,13,1,2,45,89,89]
    }), isFalse);
    expect (s.validate({
      'array': [15,13,1,2,45]
    }), isTrue);
  });
  
  test ('Schema bool test', () {
    var json = {
      'id': 'bool_test',
      'type': 'object',
      'fields': {
        'bool': {
          'type': 'bool'
        }
      }
    };
    var s = new Schema (json);
    
    expect (s.validate({
      'bool': true
    }), isTrue);
    expect (s.validate({
      'bool': false
    }), isTrue);
    expect (s.validate({
      'bool': 15
    }), isFalse);
  });
  
  test ('Schema int test', () {
    var json = {
            'id': 'int_test',
            'type': 'object',
            'fields': {
              'integer': {
                'type': 'int'
              }
            }
    };
    var s = new Schema(json);
    expect (s.validate({
      'integer': 15
    }), isTrue);
    expect (s.validate({
      'integer': 15.0
    }), isFalse);
    
    json['fields']['integer']['min'] = 0;
    json['fields']['integer']['max'] = 100;
    expect (s.validate({
      'integer': 15
    }), isTrue);
    expect (s.validate({
      'integer': -15
    }), isFalse);
    expect (s.validate({
      'integer': 100
    }), isTrue);
    expect (s.validate({
      'integer': 115
    }), isFalse);
  });
  
  test ('SJSType', () {
    expect (SJSEnum.regEnum.hasMatch('enum(a,b,c)'), isTrue);
    expect (SJSEnum.regEnum.hasMatch('enum(a)'), isTrue);
    expect (SJSEnum.regEnum.hasMatch('enum()'), isFalse);
  });
}